import requests

from ..constants import DOCUMENTS_URL, REVISIONS_URL, INDEX_FIELDS_URL, STATUS_URL


class DocumentService:
    def __init__(self, vault):
        """
        :param vault: Vault
        """
        self.vault = vault

    def get_documents(self, query_string):
        """
        get documents by query string parameter
        :param query_string: string, example: "folderPath='/test'"
        :return: dict
        """
        endpoint = DOCUMENTS_URL + '?q=' + query_string
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def get_document(self, document_id):
        """
        get document by documentId
        :param document_id: string uuid4
        :return: dict
        """
        endpoint = DOCUMENTS_URL + '/' + document_id
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def get_document_revisions(self, document_id):
        """
        get revisions of a document by documentId
        :param document_id: string uuid4
        :return: dict
        """
        endpoint = DOCUMENTS_URL + '/' + document_id + '/' + REVISIONS_URL
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def get_document_revision(self, document_id, revision_id):
        """
        get a specific revision of a document by documentId and revisionId
        :param document_id: string uuid4
        :param revision_id: string uuid4
        :return: dict
        """
        endpoint = DOCUMENTS_URL + '/' + document_id + '/' + REVISIONS_URL + '/' + revision_id
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def get_document_index_fields(self, document_id):
        """
        get indexfields for a document
        :param document_id: string uuid4
        :return: dict
        """
        endpoint = DOCUMENTS_URL + '/' + document_id + '/' + INDEX_FIELDS_URL
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def get_document_index_field(self, document_id, field_id):
        """
        get a document index field by documentId and fieldId
        :param document_id: string uuid4
        :param field_id: string uuid4
        :return: dict
        """
        endpoint = DOCUMENTS_URL + '/' + document_id + '/' + INDEX_FIELDS_URL + '/' + field_id
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def get_document_revision_index_fields(self, document_id, revision_id):
        """
        get index fields for a document revision
        :param document_id: string uuid4
        :param revision_id: string uuid4
        :return: dict
        """
        endpoint = DOCUMENTS_URL + '/' + document_id + '/' + REVISIONS_URL + '/' + revision_id + '/' + INDEX_FIELDS_URL
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def get_document_revision_index_field(self, document_id, revision_id, field_id):
        """
        get a index field of a revision of a document by documentId and revisionId
        :param document_id: string uuid4
        :param revision_id: string uuid4
        :param field_id: string uuid4
        :return: dict
        """
        endpoint = DOCUMENTS_URL + '/' + document_id + '/' + REVISIONS_URL + '/' + revision_id + '/' + INDEX_FIELDS_URL + '/' + field_id

        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def update_document_index_fields(self, document_id, fields_dict):
        """
        update one to many document index fields
        :param document_id: string uuid4
        :param fields_dict: string dict, example: "{'test': 'changed value'}"
        :return: dict
        """
        endpoint = DOCUMENTS_URL + '/' + document_id + '/' + INDEX_FIELDS_URL
        request_url = self.vault.base_url + endpoint
        fields = {'indexfields': fields_dict}
        headers = self.vault.get_auth_headers()
        resp = requests.put(request_url, headers=headers, data=fields).json()

        return resp

    def update_document_index_field(self, document_id, field_id, value):
        """
        updates a document index field
        :param document_id: string uuid4
        :param field_id: string uuid4
        :param value: string
        :return: dict
        """
        endpoint = DOCUMENTS_URL + '/' + document_id + '/' + INDEX_FIELDS_URL + '/' + field_id
        request_url = self.vault.base_url + endpoint
        fields = {'value': value}
        headers = self.vault.get_auth_headers()
        resp = requests.put(request_url, headers=headers, data=fields).json()

        return resp

    def new_document(self, folder_id, document_state, name, description, revision, file_name):
        """
        creates a document object with no file attached. first step in file upload process.
        :param folder_id: string uuid4
        :param document_state: int
        :param name: string
        :param description: string
        :param revision: string
        :param file_name: string
        :return:
        """
        endpoint = DOCUMENTS_URL
        request_url = self.vault.base_url + endpoint

        payload = {
            'folderId': folder_id,
            'documentState': document_state,
            'name': name,
            'description': description,
            'revision': revision,
            'allowNoFile': True,
            'fileLength': 0,
            'fileName': file_name,
            'indexFields': '{}'
        }

        headers = self.vault.get_auth_headers()
        resp = requests.post(request_url, data=payload, headers=headers).json()

        return resp

    def delete_document(self, document_id):
        """
        :param document_id: string uuid4
        :return: dict
        """
        endpoint = DOCUMENTS_URL + '/' + document_id
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.delete(request_url, headers=headers).json()

        return resp

    def update_document_check_in_status(self, document_id, check_in):
        """
        accepted values: 0 (Checked In), 1 (Checked Out)
        :param document_id: uuid4 formatted str
        :param check_in: int
        :return: dict
        """
        endpoint = DOCUMENTS_URL + '/' + document_id + '/' + STATUS_URL
        params = dict(checkIn=check_in)
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.put(request_url, headers=headers, json=params).json()

        return resp



import datetime
from .constants import TOKEN_EXPIRY_TIMEZONE
from pytz import timezone


def get_token_expiration(expires):
    """
    :param expires: int, seconds value
    :return: datetime
    """
    tz = timezone(TOKEN_EXPIRY_TIMEZONE)
    utc_now = datetime.datetime.now(tz)
    expiration_date = utc_now + datetime.timedelta(seconds=expires)

    return expiration_date



import requests
from ..constants import FILES_URL


class FileService:
    def __init__(self, vault):
        """
        :param vault: Vault
        """
        self.vault = vault

    def get_file_stream(self, file_id):
        """
        request a file stream by file_id
        :param file_id: string uuid4
        :return: stream
        """
        endpoint = FILES_URL + '/' + file_id
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        stream = requests.get(request_url, headers=headers, stream=True)  # get file stream

        return stream

    def get_file_stream_by_search(self, query):
        """
        request a file stream by query
        :param query: string, example: "id='8fefa9b6-56fa-e811-a995-a3d452a1c2f6'"
        :return: stream
        """
        endpoint = FILES_URL + '?q=' + query
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        stream = requests.get(request_url, headers=headers, stream=True)  # get file stream

        return stream

    def file_upload(self, document_id, name, revision, change_reason, check_in_state, index_fields, file_name,
                    file_path, check_in=True):
        """
        use a file path to upload the file stream
        :param document_id: string uuid4
        :param name: string
        :param revision: string
        :param change_reason: string
        :param check_in_state: string
        :param index_fields: string dict, example: "{'testFIELD': 'the value'}"
        :param file_name: string
        :param file_path: string
        :param check_in: bool, default: True
        :return: dict
        """
        request_url = self.vault.base_url + FILES_URL
        headers = self.vault.get_auth_headers()

        payload = {
            'documentId': document_id,
            'name': name,
            'revision': revision,
            'changeReason': change_reason,
            'checkInDocumentState': check_in_state,
            'indexFields': index_fields,
            'fileName': file_name,
            'checkIn': check_in
        }

        with open(file_path, 'rb') as file_stream:  # open file stream
            files = {'fileUpload': (file_name, file_stream, 'application/octet-stream')}
            resp = requests.post(request_url, headers=headers, data=payload, files=files).json()

        return resp

    def file_stream_upload(self, document_id, name, revision, change_reason, check_in_state, index_fields, file_name,
                           file_stream, check_in):
        """
        provide the file stream for the file upload
        :param document_id: string uuid4
        :param name: string
        :param revision: string
        :param change_reason: string
        :param check_in_state: string
        :param index_fields: string dict, example: "{'testFIELD': 'the value'}"
        :param file_name: string
        :param file_stream: bytes or BufferedReader (BytesIO.getvalue(), or open)
        :param check_in: bool
        :return: dict
        """
        request_url = self.vault.base_url + FILES_URL
        headers = self.vault.get_auth_headers()

        payload = {
            'documentId': document_id,
            'name': name,
            'revision': revision,
            'changeReason': change_reason,
            'checkInDocumentState': check_in_state,
            'indexFields': index_fields,
            'fileName': file_name,
            'checkIn': check_in
        }

        files = {'fileUpload': (file_name, file_stream, 'application/octet-stream')}
        resp = requests.post(request_url, headers=headers, data=payload, files=files).json()

        return resp



import requests
from ..constants import FOLDERS_URL, DOCUMENTS_URL, INDEX_FIELDS_URL, SELECT_OPTIONS_URL, WORKFLOW_ASSIGNMENTS_URL


class FolderService:
    def __init__(self, vault):
        """
        :param vault: Vault
        """
        self.vault = vault

    def get_folder_search(self, folder_path):
        """
        search for a folder by folderPath
        :param folder_path: string
        :return: dict
        """
        endpoint = FOLDERS_URL + '?folderPath=' + folder_path
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def get_folder(self, folder_id):
        """
        get a folder by id
        :param folder_id: string uuid4
        :return: dict
        """
        endpoint = FOLDERS_URL + '/' + folder_id
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def get_sub_folders(self, folder_id):
        """
        get all sub folders of a folder by id
        :param folder_id: string uuid4
        :return: dict
        """
        endpoint = FOLDERS_URL + '/' + folder_id + '/' + FOLDERS_URL
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def get_folder_documents(self, folder_id):
        """
        get all documents of a folder by id
        :param folder_id: string uuid4
        :return: dict
        """
        endpoint = FOLDERS_URL + '/' + folder_id + '/' + DOCUMENTS_URL
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def get_folder_index_fields(self, folder_id):
        """
        get all index fields of a folder by id
        :param folder_id: string uuid4
        :return: dict
        """
        endpoint = FOLDERS_URL + '/' + folder_id + '/' + INDEX_FIELDS_URL
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def get_folder_index_field(self, folder_id, field_id):
        """
        get a folder index field
        :param folder_id: string uuid4
        :param field_id: string uuid4
        :return: dict
        """
        endpoint = FOLDERS_URL + '/' + folder_id + '/' + INDEX_FIELDS_URL + '/' + field_id
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def get_folder_index_field_select_options(self, folder_id, field_id):
        """
        get select options for a folder index field
        TODO: unittest
        :param folder_id: string uuid4
        :param field_id: string uuid4
        :return: dict
        """
        endpoint = FOLDERS_URL + '/' + folder_id + '/' + INDEX_FIELDS_URL + '/' + field_id + '/' + SELECT_OPTIONS_URL
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def new_folder(self, name, description, allow_revisions):
        """
        creates a new folder
        :param name: string
        :param description: string
        :param allow_revisions: bool
        :return: dict
        """
        request_url = self.vault.base_url + FOLDERS_URL
        headers = self.vault.get_auth_headers()

        payload = {
            'name': name,
            'description': description,
            'allowRevisions': allow_revisions
        }

        resp = requests.post(request_url, data=payload, headers=headers).json()

        return resp

    def new_sub_folder(self, folder_id, name, description, allow_revisions):
        """
        creates a new sub folder
        :param folder_id: string uuid4
        :param name: string
        :param description: string
        :param allow_revisions: bool
        :return: dict
        """
        request_url = self.vault.base_url + FOLDERS_URL + '/' + folder_id
        headers = self.vault.get_auth_headers()

        payload = {
            'name': name,
            'description': description,
            'allowRevisions': allow_revisions
        }

        resp = requests.post(request_url, data=payload, headers=headers).json()

        return resp

    def update_folder_index_field(self, folder_id, field_id, required, default_value, query_display_field='',
                                  query_value_field='', query_id='00000000-0000-0000-0000-000000000000',
                                  drop_down_list_id='00000000-0000-0000-0000-000000000000'):
        """
        update a folder index field
        :param folder_id: string uuid4
        :param field_id: string uuid4
        :param required: bool
        :param default_value: string
        :param query_display_field: string
        :param query_value_field: string
        :param query_id: string uuid4
        :param drop_down_list_id: string uuid4
        :return: dict
        """
        endpoint = 'folders/' + folder_id + '/indexfields/' + field_id
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()

        payload = {
            'dropDownListId': drop_down_list_id,
            'queryId': query_id,
            'queryValueField': query_value_field,
            'queryDisplayField': query_display_field,
            'required': required,
            'defaultValue': default_value
        }

        resp = requests.put(request_url, data=payload, headers=headers).json()

        return resp

    def reassign_workflow_relations(self, folder_id, wf_id, event_type, apply_to_children, include_this_folder):
        """
        re assign workflow relations for a folder
        TODO: event_type documentation and unittest
        :param folder_id: string uuid4
        :param wf_id: string uuid4
        :param event_type: int
        :param apply_to_children: bool
        :param include_this_folder: bool
        :return: dict
        """
        endpoint = 'folders/' + folder_id + '/' + WORKFLOW_ASSIGNMENTS_URL
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()

        payload = {
            'wfId': wf_id,
            'eventType': event_type,
            'applyToChildren': apply_to_children,
            'includeThisFolder': include_this_folder
        }

        resp = requests.put(request_url, data=payload, headers=headers).json()

        return resp



import requests
from ..constants import SITES_URL, USERS_URL


class SiteService:
    def __init__(self, vault):
        self.vault = vault

    def get_sites(self):
        """
        get all sites
        :return: dict
        """
        request_url = self.vault.base_url + SITES_URL
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def get_site(self, site_id):
        """
        get a site by id
        :param site_id: string uuid4
        :return: dict
        """
        endpoint = SITES_URL + '/' + site_id
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def get_site_users(self, site_id):
        """
        get all users of a site
        :param site_id: string uuid4
        :return: dict
        """
        endpoint = SITES_URL + '/' + site_id + '/' + USERS_URL
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def create_site(self, name, description):
        """
        create a new site
        :param name: string
        :param description: string
        :return: dict
        """
        request_url = self.vault.base_url + SITES_URL
        headers = self.vault.get_auth_headers()

        payload = {
            'name': name,
            'description': description
        }

        resp = requests.post(request_url, headers=headers, data=payload).json()

        return resp

    def create_site_user(self, site_id, user_id, first_name, last_name, email, password, must_change_password=False,
                         send_email=False, password_never_expires=True):
        """
        creates a new user for a site
        :param site_id: string uuid4
        :param user_id: string uuid4
        :param first_name: string
        :param last_name: string
        :param email: string
        :param password: string
        :param must_change_password: bool, default False
        :param send_email: bool, default False
        :param password_never_expires: bool, default True
        :return: dict
        """
        endpoint = SITES_URL + '/' + site_id + '/' + USERS_URL
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()

        payload = {
            'userId': user_id,
            'firstName': first_name,
            'lastName': last_name,
            'emailAddress': email,
            'password': password,
            'mustChangePassword': must_change_password,
            'sendEmail': send_email,
            'passwordNeverExpires': password_never_expires
        }

        resp = requests.post(request_url, headers=headers, data=payload).json()

        return resp



import setuptools

with open('README.md', 'r') as readme:
    long_description = readme.read()

setuptools.setup(
    name='vvrest',
    version='1.7.0',
    author='Jared Runyon',
    author_email='jared.runyon@visualvault.com',
    maintainer='Jared Runyon',
    maintainer_email='jared.runyon@visualvault.com',
    description='A Python REST client library for accessing VisualVault.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/VisualVault/vvrest',
    packages=setuptools.find_packages(exclude=['tests']),
    install_requires=['requests'],
    classifiers=[
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



GRANT_TYPE_PASSWORD = 'password'
GRANT_TYPE_REFRESH = 'refresh_token'

# url routes
TOKEN_URL = '/oauth/token'  # only route that leads with '/' (does not use Vault.base_url)
DOCUMENTS_URL = 'documents'
REVISIONS_URL = 'revisions'
INDEX_FIELDS_URL = 'indexfields'
EMAILS_URL = 'emails'
FILES_URL = 'files'
FOLDERS_URL = 'folders'
SELECT_OPTIONS_URL = 'selectoptions'
WORKFLOW_ASSIGNMENTS_URL = 'workflowassignments'
FORM_TEMPLATES_URL = 'formtemplates'
FORMS_URL = 'forms'
PDF_URL = 'PDF'
FIELDS_URL = 'fields'
FORM_INSTANCE_URL = 'forminstance'
RELATE_FORM_URL = 'relateform?relatetoid='
RELATE_DOCUMENT_URL = 'relatedocument?relatetoid='
RELATE_PROJECT_URL = 'relateproject?relatetoid='
UNRELATE_FORM_URL = 'unrelateform?relatetoid='
UNRELATE_DOCUMENT_URL = 'unrelatedocument?relatetoid='
UNRELATE_PROJECT_URL = 'unrelateproject?relatetoid='
VV_LOGIN_URL = 'vvlogin?token='
RETURN_URL = '&returnUrl=~%2fFormDetails%3fformid%3d'
HIDE_MENU_URL = '%26hidemenu%3Dtrue'
GROUPS_URL = 'groups'
USERS_URL = 'users'
SITES_URL = 'sites'
WEB_TOKEN_URL = 'webtoken'
SITE_ID_URL = '?siteId='
TOKEN_EXPIRY_TIMEZONE = 'UTC'
JWT_URL = 'getJWT'
STATUS_URL = 'status'
